@@ -523,11 +523,13 @@ class Control:
                 voltage = resistance*current/1000
 
             if voltage is not None:
-                self._signal_generator_inst.set_amplitude(voltage, unit='mVpp', source=signal_gen_num)
+                self._signal_generator_inst.set_amplitude(voltage, unit='mVpp',
+                                                          source=signal_gen_num)
 
             # frequency
             if frequency is not None:
-                self._signal_generator_inst.set_frequency(frequency, unit='Hz', source=signal_gen_num)
+                self._signal_generator_inst.set_frequency(frequency, unit='Hz',
+                                                          source=signal_gen_num)
 
 
             # offset
@@ -643,10 +645,12 @@ class Control:
         """
         bias = nan
         #try:
-        bias = self._get_sensor_val('tes_bias',
-                                    tes_channel=tes_channel,
-                                    detector_channel=detector_channel,
-                                    adc_id=adc_id, adc_channel=adc_channel)
+        bias = self._get_sensor_val(
+            'tes_bias',
+            tes_channel=tes_channel,
+            detector_channel=detector_channel,
+            adc_id=adc_id, adc_channel=adc_channel
+        )
             
         #except:
         #    print('ERROR getting TES bias')
@@ -666,11 +670,12 @@ class Control:
         """
         bias = nan
         try:
-            bias = self._get_sensor_val('squid_bias',
-                                        tes_channel=tes_channel,
-                                        detector_channel=detector_channel,
-                                        adc_id=adc_id, adc_channel=adc_channel)
-                                                   
+            bias = self._get_sensor_val(
+                'squid_bias',
+                tes_channel=tes_channel,
+                detector_channel=detector_channel,
+                adc_id=adc_id, adc_channel=adc_channel
+            )                                       
         except:
             print('ERROR getting SQUID bias')
             
@@ -691,11 +696,12 @@ class Control:
         """
         lock_point = nan
         try:
-            lock_point = self._get_sensor_val('lock_point_voltage',
-                                              tes_channel=tes_channel,
-                                              detector_channel=detector_channel,
-                                              adc_id=adc_id, adc_channel=adc_channel)
-            
+            lock_point = self._get_sensor_val(
+                'lock_point_voltage',
+                tes_channel=tes_channel,
+                detector_channel=detector_channel,
+                adc_id=adc_id, adc_channel=adc_channel
+            )
         except:
             print('ERROR getting lock point')
             
@@ -717,11 +723,12 @@ class Control:
         """
         feedback_gain = nan
         try:
-            feedback_gain = self._get_sensor_val('feedback_gain',
-                                                 tes_channel=tes_channel,
-                                                 detector_channel=detector_channel,
-                                                 adc_id=adc_id, adc_channel=adc_channel)
-            
+            feedback_gain = self._get_sensor_val(
+                'feedback_gain',
+                tes_channel=tes_channel,
+                detector_channel=detector_channel,
+                adc_id=adc_id, adc_channel=adc_channel
+            )
         except:
             print('ERROR getting feedback gain')
             
@@ -738,10 +745,12 @@ class Control:
         """
         output_offset = nan
         try:
-            output_offset = self._get_sensor_val('output_offset',
-                                                 tes_channel=tes_channel,
-                                                 detector_channel=detector_channel,
-                                                 adc_id=adc_id, adc_channel=adc_channel)
+            output_offset = self._get_sensor_val(
+                'output_offset',
+                tes_channel=tes_channel,
+                detector_channel=detector_channel,
+                adc_id=adc_id, adc_channel=adc_channel
+            )
             
         except:
             print('ERROR getting output offset')
@@ -761,11 +770,12 @@ class Control:
         """
         output_variable_gain = 1
         try:
-            output_variable_gain = self._get_sensor_val('output_gain',
-                                                        tes_channel=tes_channel,
-                                                        detector_channel=detector_channel,
-                                                        adc_id=adc_id, adc_channel=adc_channel)
-            
+            output_variable_gain = self._get_sensor_val(
+                'output_gain',
+                tes_channel=tes_channel,
+                detector_channel=detector_channel,
+                adc_id=adc_id, adc_channel=adc_channel
+            )
         except:
             print('ERROR getting output gain')
             
@@ -795,11 +805,12 @@ class Control:
         """
         preamp_variable_gain = 1
         try:
-            preamp_variable_gain = self._get_sensor_val('preamp_gain',
-                                                        tes_channel=tes_channel,
-                                                        detector_channel=detector_channel,
-                                                        adc_id=adc_id, adc_channel=adc_channel)
-            
+            preamp_variable_gain = self._get_sensor_val(
+                'preamp_gain',
+                tes_channel=tes_channel,
+                detector_channel=detector_channel,
+                adc_id=adc_id, adc_channel=adc_channel
+            )
         except:
             print('ERROR getting preamp gain')
             
@@ -1026,6 +1037,15 @@ class Control:
         output_dict['voltage'] = []
         output_dict['current'] = []
 
+
+        # check if input channel
+        is_channel_defined = False
+        if (tes_channel is not None
+            or detector_channel is not None
+            or adc_channel is not None):
+            is_channel_defined = True
+
+        
         # magnicon
         if self._signal_generator_name == 'magnicon':
 
@@ -1088,7 +1108,8 @@ class Control:
                 output_dict['shape'] = self._signal_generator_inst.get_shape(source=signal_gen_num)
 
             # source
-            if self._squid_controller_name == 'feb':
+            if (is_channel_defined
+                and self._squid_controller_name=='feb'):
 
 
                 # get readout controller ID and Channel
@@ -1130,9 +1151,9 @@ class Control:
                 else:
                     output_dict['source'] = 'none'
             
-            else:
+            #else:
                 # FIXME: default to tes if not FEB
-                output_dict['source'] = 'tes'
+            #    output_dict['source'] = 'tes'
                 
         return output_dict
 
@@ -1767,11 +1788,13 @@ class Control:
 
         # get readout controller ID and Channel
         controller_id, controller_channel = (
-            connection_utils.get_controller_info(self._connection_table,
-                                                 tes_channel=tes_channel,
-                                                 detector_channel=detector_channel,
-                                                 adc_id=adc_id, 
-                                                 adc_channel=adc_channel)
+            connection_utils.get_controller_info(
+                self._connection_table,
+                tes_channel=tes_channel,
+                detector_channel=detector_channel,
+                adc_id=adc_id, 
+                adc_channel=adc_channel
+            )
         )
               
         # ================
@@ -1788,29 +1811,37 @@ class Control:
             slot = int(feb_info[1])
             
             if self._verbose or self._debug:
-                print('INFO: Setting ' + param_name + ' to ' + str(value) + ' using FEB!')
+                print('INFO: Setting ' + param_name
+                      + ' to ' + str(value) + ' using FEB!')
             if self._debug:
-                print('DEBUG: FEB - subrack = ' + str(subrack) + ', slot = ' + str(slot) + ', channel = ' + 
-                      str(controller_channel))
+                print('DEBUG: FEB - subrack = ' + str(subrack)
+                      + ', slot = ' + str(slot) + ', channel = '
+                      + str(controller_channel))
                  
             if not self._dummy_mode:
                 if param_name == 'tes_bias':
-                    self._readout_inst.set_phonon_qet_bias(subrack, slot, controller_channel, value)
+                    self._readout_inst.set_phonon_qet_bias(
+                        subrack, slot, controller_channel, value)
             
                 elif param_name == 'squid_bias':
-                    self._readout_inst.set_phonon_squid_bias(subrack, slot, controller_channel, value)
+                    self._readout_inst.set_phonon_squid_bias(
+                        subrack, slot, controller_channel, value)
                 
                 elif param_name == 'lock_point_voltage':
-                    self._readout_inst.set_phonon_lock_point(subrack, slot, controller_channel, value)
+                    self._readout_inst.set_phonon_lock_point(
+                        subrack, slot, controller_channel, value)
                 
                 elif param_name == 'preamp_gain':
-                    self._readout_inst.set_phonon_preamp_gain(subrack, slot, controller_channel, value)
+                    self._readout_inst.set_phonon_preamp_gain(
+                        subrack, slot, controller_channel, value)
                 
                 elif param_name == 'output_offset':
-                    self._readout_inst.set_phonon_offset(subrack, slot, controller_channel, value)
+                    self._readout_inst.set_phonon_offset(
+                        subrack, slot, controller_channel, value)
                 
                 elif param_name == 'output_gain':
-                    self._readout_inst.set_phonon_output_gain(subrack, slot, controller_channel, value)
+                    self._readout_inst.set_phonon_output_gain(
+                        subrack, slot, controller_channel, value)
                 
                 elif param_name == 'feedback_polarity':
                     do_invert = False
@@ -1955,8 +1986,11 @@ class Control:
         """
 
         # check input
-        if instrument_name is None and channel_name is None and global_channel_number is None:
-            raise ValueError('ERROR: Missing channel name, global number, or instrument name')
+        if (instrument_name is None
+            and channel_name is None
+            and global_channel_number is None):
+            raise ValueError('ERROR: Missing channel name, '
+                             +'global number, or instrument name')
     
 
         # get instrument name
@@ -1964,7 +1998,9 @@ class Control:
         if instrument_name is None:
 
             # get channel table
-            channel_table = self.get_temperature_controllers_table(channel_type=channel_type)
+            channel_table = self.get_temperature_controllers_table(
+                channel_type=channel_type
+            )
 
             # query instrument name
             if channel_name is not None:
@@ -2002,14 +2038,18 @@ class Control:
         """
         
         # find instrument
-        inst = self.get_temperature_controller(channel_name=channel_name,
-                                               global_channel_number=global_channel_number,
-                                               instrument_name=instrument_name,
-                                               channel_type='resistance')
+        inst = self.get_temperature_controller(
+            channel_name=channel_name,
+            global_channel_number=global_channel_number,
+            instrument_name=instrument_name,
+            channel_type='resistance'
+        )
         
         # get temperature
-        temperature = inst.get_temperature(channel_name=channel_name,
-                                          global_channel_number=global_channel_number)
+        temperature = inst.get_temperature(
+            channel_name=channel_name,
+            global_channel_number=global_channel_number
+        )
 
        
         return temperature
@@ -2027,14 +2067,18 @@ class Control:
         """
 
         # find instrument
-        inst = self.get_temperature_controller(channel_name=channel_name,
-                                               global_channel_number=global_channel_number,
-                                               instrument_name=instrument_name,
-                                               channel_type='resistance')
+        inst = self.get_temperature_controller(
+            channel_name=channel_name,
+            global_channel_number=global_channel_number,
+            instrument_name=instrument_name,
+            channel_type='resistance'
+        )
         
         
-        resistance = inst.get_resistance(channel_name=channel_name,
-                                         global_channel_number=global_channel_number)
+        resistance = inst.get_resistance(
+            channel_name=channel_name,
+            global_channel_number=global_channel_number
+        )
 
        
         return resistance
@@ -2137,8 +2181,10 @@ class Control:
                 if mag_control_info:
                     print('Controller info for Magnicon:')
                     print('Channel list:', str(mag_control_info['channel_list']))
-                    print('Default active channel:', str(mag_control_info['default_active']))
-                    print('Reset active channel after every step:', str(mag_control_info['reset_active']))
+                    print('Default active channel:',
+                          str(mag_control_info['default_active']))
+                    print('Reset active channel after every step:',
+                          str(mag_control_info['reset_active']))
    
 
             if self._squid_controller_name == 'magnicon':
@@ -2152,9 +2198,11 @@ class Control:
         # ----------------
         if self._signal_generator_name == 'keysight':
             address = self._config.get_signal_generator_visa_address('keysight')
+            attenuation = self._config.get_signal_generator_attenuation('keysight')
             self._signal_generator_inst = (
                 KeysightFuncGenerator(address,
                                       visa_library=visa_library,
+                                      attenuation=attenuation,
                                       verbose=self._verbose,
                                       raise_errors=self._raise_errors)
             )
